/*
 * signals/surprise.c --
 *
 *	Depending on the platform you run this program on, you might
 *	see that signals may interrupt even simple assignments. The
 *	alarm system call arranges for a SIGALRM signal to be
 *	delivered to the calling process after a number of seconds.
 */

#define _POSIX_C_SOURCE 200809L

#include <stdlib.h>
#include <signal.h>
#include <stdint.h>
#include <stdio.h>
#include <inttypes.h>
#include <unistd.h>

typedef struct twins { int64_t a, b; } twins_t;
static const twins_t zeros = { 0, 0 }, ones = { 1, 1 };
static twins_t memory;

static void
handler(int signum)
{
    if (memory.a != memory.b) {
	printf("memory = {%" PRId64 ",%" PRId64 "}\n", memory.a, memory.b);
    }
    (void) alarm(1);                            /* request next alarm */
}

int
main(void)
{
    struct sigaction sa;

    sa.sa_handler = handler;
    sigemptyset(&sa.sa_mask);
    sa.sa_flags = SA_RESTART;

    if (sigaction(SIGALRM, &sa, NULL) == -1) {
        perror("sigaction");
        return EXIT_FAILURE;
    }
    
    (void) alarm(1);                            /* request first alarm */
    
    for (memory = zeros; 1; memory = zeros) {
	memory = ones;
    }
    return EXIT_SUCCESS;
}
